/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: FormGateway.java,v 1.4 2002/05/29 06:17:35 JacobK Exp $
 */
package org.enhydra.barracuda.core.comp.helper;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.exceptions.*;
import org.enhydra.barracuda.core.util.http.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.view.*;

/**
 * <p>The purpose of this servlet is simply to process requests 
 * that come from a form and redirect based on the name of the button 
 * pressed.
 */
public class FormGateway extends HttpServlet {

    //public vars...eventually, these should probably be final
    protected static Logger logger = Logger.getLogger(FormGateway.class.getName());

    //these are intentionally non-final, so that you can programatically
    //change them if need be
    public static String FORM_EXT = ".form_forward";
    public static String FORM_TARGET = "ft::";

    //-------------------- FormGateway ---------------------------
    /**
     * <p>Handle the default HttpRequest. 
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleDefault(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
    
        //iterate through all the parameters, looking for those which indicate
        //the key name is really an event handler or url
        Enumeration enum = req.getParameterNames();
        int matches = 0;
        Servlet servlet = null;
        String target = null;
        if (logger.isDebugEnabled()) logger.debug("Looking for target parameters");
        while (enum.hasMoreElements()) {
            String key = (String) enum.nextElement();
            String val = (String) req.getParameter(key);
            if (logger.isDebugEnabled()) logger.debug("key:"+key+" val:"+val);
            
            if (key.startsWith(FORM_TARGET)) {
                if (logger.isDebugEnabled()) logger.debug("(form target match)");
                target = key.substring(FORM_TARGET.length());
                matches++;
            }
        }
//csc_101901.1        target = URLRewriter.encodeURL(req, resp, target);   //csc_101701.1
        
        //this is basically checks to ensure that only one of the req parameters 
        //matches a known form handler or URL. If more than one matches, then there's 
        //no way of knowing which one it's really supposed to be delivered to, so 
        //throw an exception
        if (matches!=1) throw new ServletException ("Unable to determine destination handler");
        else {        
            //find the RequestDispatcher and forward it there
            RequestDispatcher rd = req.getRequestDispatcher(target);
            if (logger.isDebugEnabled()) logger.debug("Redirecting to url:"+target);
            if (rd!=null) {
                rd.forward(req, resp);
            } else {
                throw new ServletException ("Unable to locate RequestDispatcher");
            }
        }
    }




    //-------------------- HTTPServlet ---------------------------
    /**
     * <p>By default the GET request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the POST request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the OPTIONS request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doOptions(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the DELETE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doDelete(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the PUT request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPut(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the TRACE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doTrace(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {  
        handleDefault(req, resp);
    }
}
