/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLInputRenderer.java,v 1.9 2002/05/29 06:17:35 JacobK Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.apache.log4j.*;

/**
 * This class handles the default rendering of a list in an HTML view.
 */
public class HTMLInputRenderer extends HTMLComponentRenderer {

    protected static Logger logger = Logger.getLogger(HTMLInputRenderer.class.getName());
    
    protected EnabledHelper eh = new EnabledHelper();
        
    protected BInput bicomp = null;
    
    /**
     * The purpose of this method is to create a default Node to be used when
     * the component is not bound to any specific view. 
     *
     * @param doc the master Document which can be used to create elements
     *        from scratch
     * @param comp the component that we're dealing with for the current request
     * @param vc the view context for the current request
     * @return a default node (created from scratch)
     * @throws UnsupportedFormatException if the renderer has no default node
     */
//csc_110501.1    public Node createDefaultNode(Document doc, ViewContext vc) throws UnsupportedFormatException;
    public Node createDefaultNode(Document doc, BComponent comp, ViewContext vc) throws UnsupportedFormatException {  //csc_110501.1
        //ask the renderer to create the default Node
/*
//removed - should clone the default node        
        Element inputEl = (HTMLInputElement) doc.createElement("INPUT");
        Node defaultNode = inputEl;
*/        
        Node templateNode = vc.getTemplateNode();
        Node defaultNode = null;
        if (templateNode instanceof HTMLInputElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else {
            defaultNode = (HTMLInputElement) doc.createElement("INPUT");
        }
        
        if (logger.isInfoEnabled()) logger.info("Creating default node:"+defaultNode);
        return defaultNode;
    }

    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a list component
        if (!(comp instanceof BInput)) throw new NoSuitableRendererException("This renderer can only render BInput components; comp is of type:"+comp.getClass().getName());
        bicomp = (BInput) comp;
        Node node = view.getNode();

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //..HTMLInputElement - set the "type" and "value" attributes
        if (node instanceof HTMLInputElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLInputElement interface...");
            HTMLInputElement el = (HTMLInputElement) node;

            //set the type (if specified)
            String type = bicomp.getType();
            if (type!=null) el.setAttribute("type", type);

            //set the value (if specified)
            String value = bicomp.getValue();
            if (value!=null) el.setValue(value);
            
        } else {
            String errmsg = "Node does not implement HTMLInputElement and cannot be rendered: "+node;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }

        //finally, make sure we reflect the components enabled/disabled status
        eh.setEnabled(node, comp.isEnabled());
    }
}