/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: HTMLScriptRenderer.java,v 1.8 2002/05/29 06:17:35 JacobK Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.scripting.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.view.*;

/**
 * This class handles the default rendering script references into an HTML view.
 */
public class HTMLScriptRenderer extends DOMComponentRenderer {

    protected static Logger logger = Logger.getLogger(HTMLScriptRenderer.class.getName());

    protected EnabledHelper eh = new EnabledHelper();

    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //unlike other components, this one really doesn't care what we're bound
        //to...we do need to make sure the component is a script component
        if (!(comp instanceof BScript)) throw new NoSuitableRendererException("This renderer can only render BScript components");
        BScript bsComp = (BScript) comp;
        Node node = view.getNode();

        //sanity checks
        String jsattr = bsComp.getAttr();
        String jscmd = bsComp.getCmd();
        int mode = bsComp.getMode();
        if (jsattr==null || jscmd==null) return;
        if (!(node instanceof Element)) return;
        Element el = (Element) node;

        //now allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //note that we always want to render this script...this way if the client
        //has scripting disabled now, but re-enables it later, we will know about it
        //the next time around. If they don't have scripting enabled this can't hurt.

        //render the component
        if (!jscmd.endsWith(";")) jscmd = jscmd + "; ";
        String cur_attr = el.getAttribute(jsattr);
        if (cur_attr==null) mode = BScript.REPLACE;
        //...append
        if (mode==BScript.APPEND) {
            if (!cur_attr.endsWith(";")) cur_attr = cur_attr + "; ";
            el.setAttribute(jsattr, cur_attr+jscmd);
        //...prepend
        } else if (mode==BScript.PREPEND) {
            el.setAttribute(jsattr, jscmd+cur_attr);
        //...replace
        } else {
            el.setAttribute(jsattr, jscmd);
        }
        
        //add in BScriptResource components that are needed
        List resources = bsComp.getResources();
        if (resources!=null) {
            Iterator it = resources.iterator();
            while (it.hasNext()) {
                String scr = (String) it.next();
                BScriptResource bsr = new BScriptResource(scr);
                bsr.setView(new DefaultView(node));
                bsComp.addStepChild(bsr, true);
            }
        }

//for script components, I don't think we need to worry about whether they're enabled or not
        //finally, make sure we reflect the components enabled/disabled status
//        eh.setEnabled(node, comp.isEnabled());
    }

}