/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DispatchQueue.java,v 1.5 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;

import org.enhydra.barracuda.core.util.data.StateMap;

/**
 * This interface defines the methods needed to implement a
 * DispatchQueue
 */
public interface DispatchQueue  { //extends StateMap {

    /**
     * Does this queue require a response?
     *
     * @return true if this queue requires a response
     */    
    public boolean requiresResponse();

    /**
     * Has the response for this queue been handled?
     *
     * @return true if the response has been handled
     */    
    public boolean responseHandled();

    /**
     * Adds an event to the queue. When this happens, we first remove
     * any existing events in the queue that this event .equals(), and
     * then this event is added. This has the effect of collapsing 
     * duplicates.
     *
     * @param baseEvent the event to be added to the queue
     */
    public void addEvent(BaseEvent baseEvent);

    /**
     * get the number of events remaining in the queue
     *
     * @return the number of events remaining in the queue
     */
    public int numberOfEventsRemaining();

    /**
     * List all events remaining in the queue (Request events first, then
     * Response events)
     *
     * @return a list of all events remaining in the queue
     */
    public List listRemainingEvents();

    /**
     * get the number of events that have been processed
     *
     * @return the number of events processed in the queue
     */
    public int numberOfEventsProcessed();

    /**
     * List events which have already been processed through the queue
     *
     * @return a list of all events processed in the queue
     */
    public List listProcessedEvents();

}
