/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ListenerFactory.java,v 1.4 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;

/**
 * This interface defines the methods needed to be an 
 * ListenerFactory. 
 * 
 * Note that a factory is responsible for ensuring threadsafety
 * within the underlying listener, so the factory should either 
 * return a new instance of the listener OR keep one static 
 * synchronized instance and return that instead.
 */
public interface ListenerFactory {

    /**
     * Get an instance of the underlying BaseEventListener
     *
     * @return get an instance of the BaseEventListener
     */
    public BaseEventListener getInstance();
    
    /**
     * Return true if you want to always be notified, even when an event has 
     * been handled by someone else. Return false (default) when you only want 
     * to notified if the event hasn't been handled already. 
     *
     * Typically, only logging type listeners would return true.
     *
     * @return return true to be notified of the event even if it's already 
     *        been handled
     */
    public boolean notifyAlways();

    /**
     * Get the Listener ID associated with this class of listener. This will
     * generally either be the class name of the listener that the factory 
     * creates
     *
     * @return the listener ID that describes this factory
     */
    public String getListenerID();

}
