/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Polymorphic.java,v 1.4 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;

/**
 * This interface indicates an Event is Polymorphic, in
 * the sense that firing it causes parent events to be fired 
 * first. For instance, if MyEvent extends PolymorphicEvent,
 * firing MyEvent first causes a PolymorphicEvent to be fired
 * because MyEvent _is_ a PolymorphicEvent. Kapish?
 *
 * This type of functionality makes it possible to install
 * listeners on a parent event and be notified when any of the 
 * child events are fired...this is sometimes very useful.
 * 
 * This interface has kind of the opposite meaning of the 
 * Exception interface. Events should never indicate both 
 * Exceptional and Polymorphic (and can't because both of these 
 * interfaces define describeEventChainingStrategy())
 */
public interface Polymorphic {
    /**
     * Describe the event chaining stategy. This method really
     * serves to ensure that objects cannot implement BOTH
     * Polymorphic and Exceptional (it's got to be one or the other)
     *
     * @return string describing the event chain strategy
     */
    public String describeEventChainingStrategy();
}
