/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: BTemplateViewHandler.java,v 1.11 2002/06/16 17:56:41 JacobK Exp $
 */
package org.enhydra.barracuda.core.event.helper;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.exceptions.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.util.http.*;
import org.apache.log4j.*;

/**
 * <p>A custom implementation of the default view handler tailored for
 * template components. All you have to do is instantiate the class
 * with the name of the template and a reference to the model; it takes
 * care of everything else.
 */
public abstract class BTemplateViewHandler extends DefaultViewHandler {

    protected BTemplate templateComp = null;    //csc_041102.1

    //-------------------- DefaultViewHandler --------------------
    /**
     * Return an instance of the template model; you can either return
     * a single instance of a TemplateModel, -OR- you can return a
     * List of TemplateModels
     */
//csc_021102.1    public abstract TemplateModel getTemplateModel();
    public abstract Object getTemplateModels();     //csc_021102.1

    /**
     * Return an instance of the template class (must implement Document)
     */
    public abstract Class getTemplateClass();

    //csc_041102.1 - added
    /**
     * Provide a handle to the underlying BTemplate component
     */
    public BTemplate getBTemplate() {
        return templateComp;
    }

    /**
     * Generate the view
     */
    public Document handleViewEvent(BComponent root, ViewContext vc) throws EventException, ServletException, IOException {
        //load the localized DOM template
        //sam_061602.1_start
        //Document page = DefaultDOMLoader.getGlobalInstance().getDOM(getTemplateClass());
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(getTemplateClass(),  vc.getViewCapabilities());
        //sam_061602.1_end

        //create a template component and bind it to the views
//csc_021102.1_start
//        BTemplate templateComp = new BTemplate(getTemplateModel());
//csc_041102.1        BTemplate templateComp = new BTemplate();
        templateComp = new BTemplate();     //csc_041102.1
        Object o = getTemplateModels();
        if (o instanceof TemplateModel) templateComp.addModel((TemplateModel) o);
        else if (o instanceof List) templateComp.addModels((List) o);
//jkf_022102.1_start
        else if (o == null) { }
//jkf_022102.1_end
        else throw new RuntimeException("Fatal err: Model must return either TemplateModel or List");
//csc_021102.1_end
        templateComp.setView(new DefaultTemplateView(page.getDocumentElement()));
        root.addChild(templateComp);

        //return the page
        return page;
    }
}
