/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: UnableToValidateException.java,v 1.4 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.io.*;
import java.util.*;

import org.enhydra.barracuda.core.util.exceptions.NestableException;

/**
 * This exception indicates the Validator was not able to validate
 * the incoming value. Usually thrown due to a type mismatch (ie. the 
 * validator is expecting a number and it gets an alphanumeric string 
 * or something like that), if this occurs it probably indicates you are 
 * improperly using a validator.
 */
public class UnableToValidateException extends ValidationException {

    protected List subExceptions = null;

    /**
     * The noargs public contructor for UnableToValidateException
     */
    public UnableToValidateException () {super(null);}

    /**
     * The public contructor for UnableToValidateException
     *
     * @param source the object which caused this error (usually a
     *        FormElement)
     */
    public UnableToValidateException (Object source) {super(source, null);}

    /**
     * The public contructor for UnableToValidateException
     *
     * @param source the object which caused this error (usually a
     *        FormElement)
     * @param s a String describing the exception
     */
    public UnableToValidateException (Object source, String s) {super(source, s, null);}

    /**
     * The public contructor for UnableToValidateException
     *
     * @param source the object which caused this error (usually a
     *        FormElement)
     * @param s a String describing the exception
     * @param iError the Error code associated with this exception
     */
    public UnableToValidateException (Object source, String s, Exception ibaseException) {super(source, s, ibaseException);}

}
