/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: StringUtil.java,v 1.1 2002/06/11 01:19:02 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util;

import org.apache.log4j.*;

/**
 * Simple utility functions that work on Strings
 */
public class StringUtil {

    //public constants
    protected static Logger logger = Logger.getLogger(StringUtil.class.getName());

    /**
     * Replace all occurences of a pattern in a String with a new pattern
     *
     * @param sourceStr the source string
     * @param oldPattern the old pattern
     * @param newPattern the new pattern
     * @return an adjusted String
     */
    public static String replace(String sourceStr, String oldPattern, String newPattern) {
        if (sourceStr!=null && sourceStr.length()<1) return sourceStr;  //no way we can match on this, so just return the source str
        String s = _replace(sourceStr, oldPattern, newPattern);
        if (s!=null && s.length()<1) {
            return (newPattern==null ? null : s);
        } else {
            return s;
        }
    }
    private static String _replace(String sourceStr, String oldPattern, String newPattern) {
        //eliminate the obvious
        if (sourceStr==null) {
            if (oldPattern==null) return newPattern;
            else return sourceStr;
        }
        else if (oldPattern==null) return sourceStr;
        if (newPattern==null) newPattern = "";

        //see if the pattern exists
        int i = sourceStr.indexOf(oldPattern);
        if (i<0) return sourceStr;
        else return sourceStr.substring(0,i)+newPattern+_replace(sourceStr.substring(i+oldPattern.length()),oldPattern,newPattern);
    }

}