/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: Param.java,v 1.2 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.data;

import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.http.*;

/**
 * <p>Define a Param object as having key and value properties
 * (both of which are Strings)
 */
public class Param {

    String key = null;
    String value = null;

    /**
     * Create an empty Param
     */
    public Param() {
        this(null, null);
    }

    /**
     * Create a Param for a given key/value
     *
     * @param key the param key
     * @param value the param value
     */
    public Param(String ikey, String ivalue) {
        setKey(ikey);
        setValue(ivalue);
    }

    /**
     * Set the key
     *
     * @param key the param key
     */
    public void setKey(String ikey) {
        key = ikey;
    }

    /**
     * Get the key
     *
     * @return the param key
     */
    public String getKey() {
        return key;
    }

    /**
     * Set the value
     *
     * @param value the param value
     */
    public void setValue(String ivalue) {
        value = ivalue;
    }

    /**
     * Get the value
     *
     * @return the param value
     */
    public String getValue() {
        return value;
    }

    /**
     * Check for equality (if both key & value match)
     *
     * @param o the object we are comparing against
     * @return true if the objects are equal
     */
    public boolean equals(Object o) {
        if (o==null) return false;
        if (o instanceof Param) {
            Param p = (Param) o;
            if (key==null && p.key!=null) return false;
            if (value==null && p.value!=null) return false;
            else return (key.equals(p.key) && value.equals(p.value));
        } else {
            return false;
        }
    }

    /**
     * Return a string representation of the param
     */
    public String toString() {
        return "Param {k:"+key+" v:"+value+"}";
    }
}
