/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ExceptionUtil.java,v 1.4 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.exceptions;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Exception utilities
 */
public class ExceptionUtil {

    /**
     * log an exception as HTML
     *
     * @param out the target PrintWriter
     * @param e the nestable exception to dump
     * @param req the servlet request that spawned all this
     */
    public static void logExceptionAsHTML (PrintWriter out, NestableException e, HttpServletRequest req) {
        out.println("<html>");
        out.println("<head>");
        out.println("<title>Unexpected exception!</title>");
        out.println("</head>");
        out.println("<body>");
        
        out.println("<h1>Error: EventException</h1>");
        out.println("<p>There was an unexpected error while servicing this request...please contact your application administrator and notify them of the problem.");
        out.println("<p>Error dispatching request: "+e.getMessage());
        out.println("<p>Exception:<br>");
        e.printStackTrace(out);
        Exception rootException = NestableException.getRootException(e);
        if (rootException!=e) {
            out.println("<p>Root Exception:<br>");
            rootException.printStackTrace(out);
        }
        
        out.println("<p>Parameters:");
        Enumeration enum = req.getParameterNames();
        while (enum.hasMoreElements()) {
            String key = (String) enum.nextElement();
            String val = (String) req.getParameter(key);
            out.println("<br>&nbsp;&nbsp;&nbsp;&nbsp;key:"+key+" value:"+val);
        }

        out.println("<br>RequestURI:"+req.getRequestURI());
        out.println("<br>ServletPath:"+req.getServletPath());
        out.println("<br>PathInfo:"+req.getPathInfo());
        out.println("<br>PathTranslated:"+req.getPathTranslated());
    
        out.println("</body>");
        out.println("</html>");
    }

}
