/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MainScreen.java,v 1.13 2002/05/29 06:17:36 JacobK Exp $
 */
package org.enhydra.barracuda.examples.ex1;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.helper.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.http.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.examples.ex1.events.*;
import org.enhydra.barracuda.examples.ex1.xmlc.*;


/**
 * Event handlers (both Controller and View) for the 
 * Main screen.
 */
public class MainScreen extends DefaultEventGateway {

    //public constants
    protected static Logger logger = Logger.getLogger(MainScreen.class.getName());

    //this defines the various event handlers
    private ListenerFactory getMainFactory =               new EventForwardingFactory(new RenderMainScreen());
    private ListenerFactory expireSessionFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new ExpireSessionHandler();}        public String getListenerID() {return getID(ExpireSessionHandler.class);}};                
    private ListenerFactory renderMainFactory =            new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderMainScreenHandler();}    public String getListenerID() {return getID(RenderMainScreenHandler.class);}};                

    //private vars
    private GetMainScreen lnkGetMainScreen;                //added by TJ to show association
    private ExpireSession lnkExpireSession;                //added by TJ to show association
    private RenderMainScreen lnkRenderMainScreen;            //added by TJ to show association

    /**
     * Public constructor
     */
    public MainScreen() {
        //specify who's interested in what
        specifyLocalEventInterests(getMainFactory, GetMainScreen.class);
        specifyLocalEventInterests(expireSessionFactory, ExpireSession.class);
        specifyLocalEventInterests(renderMainFactory, RenderMainScreen.class);
    }


    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * ExpireSessionHandler - this is where we handle any ExpireSession
     * events. Afterwards, redirect flow to the Main screen (which won't
     * work because the session information is gone now, so instead you'll
     * end up at the Login screen as you really should.)
     */
    class ExpireSessionHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            BaseEvent event = context.getEvent();
            HttpServletRequest req = context.getRequest();
            DispatchQueue queue = context.getQueue();
            logger.debug("Expire session");

            //expire the session info
            SessionServices.getSession(req).invalidate();

            //in this case, we're not really doing anything special, so just
            //redirect to the RenderMain view (note, this won't actually happen,
            //because the Utilities class is ensuring that without session info 
            //we always get redirected to the Login screen...but we shouldn't have 
            //to know about that detail here)
            BaseEvent newEvent = new GetMainScreen();
            newEvent.setSource(event);
            queue.addEvent(newEvent);
        }
    }
    
    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * RenderMainScreenHandler - this is where we handle any RenderMainScreen 
     * event and actually generate the view. We check the session info 
     * to determine how many times they've been here, as well as to see
     * if they were automatically logged in.
     */
    class RenderMainScreenHandler extends BTemplateViewHandler {
        public Object getTemplateModels() {return new LocalTemplateModel();}
        public Class getTemplateClass() {return MainHTML.class;}
    }
    
    
    //------------------------------------------------------------
    //                Components - TemplateModel
    //------------------------------------------------------------
    /**
     * LocalTemplateModel
     */
    class LocalTemplateModel extends AbstractTemplateModel {
        boolean gotState = false;
        int numberOfVisits = -1;
        boolean autoLogin = false;
    
        //register the model by name
        public String getName() {return "Ex1_Main";}

        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            if (key.equals("Visits")) {
                checkState();
                return numberOfVisits+" time"+(numberOfVisits>1 ? "s" : "");
            } else if (key.equals("AutoLogin")) {
                checkState();
                if (autoLogin) return new BComponent(); //visible (go ahead and show the node)
                else return null;                       //null (skip it)
            } else if (key.equals("SecretTip")) {
                checkState();
                if (numberOfVisits>3) return new BComponent(); //visible (go ahead and show the node)
                else return null;                       //null (skip it)
            } else if (key.equals("GetLoginLink")) {
                return new BAction(new GetLoginScreen());
            } else if (key.equals("ExpireSessionLink")) {
                return new BAction(new ExpireSession());
            } else {
                return super.getItem(key);
            }
        }
        
        private void checkState() {
            if (gotState) return;
            ViewContext vc = getViewContext();
            //...see how many times we've been here...
            HttpSession session = SessionServices.getSession(vc);
            numberOfVisits = 0;
            try {numberOfVisits = Integer.parseInt(""+session.getAttribute(LoginServices.NUMBER_OF_VISITS).toString());}
            catch (Exception e) {}
            session.setAttribute(LoginServices.NUMBER_OF_VISITS, ""+(++numberOfVisits));
            //..see if we auto-logged-in
            autoLogin = new Boolean(""+session.getAttribute(LoginServices.AUTO_LOGIN)).booleanValue();
            gotState = true;
        }
    }
}

