/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SampleViewHandler.java,v 1.10 2002/06/12 21:09:15 cryd0221 Exp $
 */
package org.enhydra.barracuda.examples.ex4;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.examples.ex4.events.*;


/**
 * A sample view event handler
 */
public class SampleViewHandler extends DefaultListenerFactory {

    public static final String HANDLED_BY = "HandledBy";

    //define the necessary factory methods
    public BaseEventListener getInstance() {
        return new RenderHandler();
    }            
    public String getListenerID() {
        return getID(RenderHandler.class);
    }

    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * RenderHandler - handle the request to render the test 
     * screen
     */
    class RenderHandler extends DefaultBaseEventListener {
        public void handleViewEvent(ViewEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            BaseEvent event = context.getEvent();
            HttpServletResponse resp = context.getResponse();

            //csc_061202.1 - added    
            //set the caching hdrs (allow this page to be cached by browser)
            resp.setHeader("Cache-Control","max-age=0");
            resp.setDateHeader("Last-Modified", System.currentTimeMillis());

            //build the response
            resp.setContentType("text/html");
            PrintWriter out = resp.getWriter();
            
            //header            
            String title = "Simple Deployment Descriptor Test Screen";
            out.println ("<html>");
            out.println ("  <head>");
            out.println ("    <title>"+title+"</title>");
            out.println ("  </head>");
            out.println ("  <body>");
            out.println ("    <font face=\"Arial\">");

            //body
            out.println ("      <p>"+title);
            out.println ("      <p>Handled by:"+context.getState(HANDLED_BY));
            
            //footer
            out.println ("    </font>");
            out.println ("  </body>");
            out.println ("</html>");
        }
    }
}

