package org.enhydra.barracuda.tutorials.comp;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.tutorials.xmlc.*;

/**
 * Here's an example of HelloWorld using the BTemplate
 * component
 */
public class HelloWorld2 extends ComponentGateway {

    //xmlc factory (ok as static because its threadsafe)
     private static XMLCFactory xmlcFactory = null;

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) {
        //load the DOM object
        if (xmlcFactory==null) xmlcFactory = new XMLCStdFactory(this.getClass().getClassLoader(), new StreamXMLCLogger());
        XMLObject page = xmlcFactory.create(HelloWorld2HTML.class);

        //create the component and bind it to the view
        BTemplate templateComp = new BTemplate();
        templateComp.setView(new DefaultTemplateView(page.getElementById("HelloWorld")));

        //now specify the models
        templateComp.addModel(new HelloWorldModel());
        String[] items = new String[] {"Cabbage", "Carrots", "Pickles", "Sugar", "Wheaties", "Flour", "Potatoes"};
        String[] qtys =  new String[] {"3 heads", "1 bag", "1 jar", "5 lbs", "3 bxs", "50 lbs", "25 lbs"};
        templateComp.addModel(new GroceriesModel(items, qtys));
        
        //add the component to the root
        root.addChild(templateComp);
    
        //return the page
        return page;
    }

    //--------------- GroceriesModel -------------------------
    /**
     * Groceries Model
     */
    class GroceriesModel extends AbstractTemplateModel implements IterativeModel {

        String[] items = null;
        String[] qtys = null;
        int cntr = -1;

        //constructor
        public GroceriesModel(String[] iitems, String[] iqtys) {
            items = iitems;
            qtys = iqtys;
        }

        //register the model by name
        public String getName() {return "Groceries";}

        //provide items by key
        public Object getItem(String key) {
            if (key.equals("Item")) return items[cntr];
            else if (key.equals("Qty")) return qtys[cntr];
            else return super.getItem(key);
        }

        //prepare for iteration
        public void preIterate() {
            cntr = -1;
        }
        
        //return true if there are more rows in the data model
        public boolean hasNext() {
            return (cntr<(items.length-1));
        }
        
        //load the next row in the model
        public void loadNext() {
            cntr++;
        }

        //cleanup after iteration
        public void postIterate() {}
    }

    //--------------- HelloWorldModel ------------------------
    /**
     * HelloWorldModel
     */
    class HelloWorldModel extends AbstractTemplateModel {
    
        //register the model by name
        public String getName() {
            return "HelloWorld";
        }
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            if (key.equals("Title")) return "Hello World 2 example";
            else if (key.equals("Descr")) return "This example illustrates how we can use directives in the template to iterate over data structures to form structures like lists and tables.";
            else if (key.equals("Hello")) return "Hello World! Hi Ma! Hi Pa!";
            else if (key.equals("Footer")) return HelloWorld2Footer.getNode(vc);
            else return super.getItem(key);
        }
    }
}
