/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *  @author     christianc@lutris.com
 *
 * Changes
 * -----------------------
 *
 * rtl_20010612 - Modified to use HelloWorld3*XML.class instead of HelloWorld3*HTML.class
 *
 */
package org.enhydra.barracuda.tutorials.comp;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.tutorials.xmlc.*;

/**
 * Here's an example of HelloWorld using the BTemplate
 * component
 */
public class HelloWorld3 extends ComponentGateway {

    //xmlc factory (ok as static because its threadsafe)
     private static XMLCFactory xmlcFactory = null;

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest.
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) {
        //figure out which page we're after by looking for an optional
        //page parameter in the Req
        String pageNo = req.getParameter("page");
        if (pageNo==null) pageNo = "1";
        boolean useIDTemplate = (!pageNo.equals("1"));
        // rtl_20010612 Class pageCl = HelloWorld3HTML.class;
        // rtl_20010612 if (useIDTemplate) pageCl = HelloWorld3aHTML.class;
        Class pageCl = HelloWorld3XML.class;
        if (useIDTemplate) pageCl = HelloWorld3aXML.class;

        //load the DOM object
        if (xmlcFactory==null) xmlcFactory = new XMLCStdFactory(this.getClass().getClassLoader(), new StreamXMLCLogger());
        XMLObject page = xmlcFactory.create(pageCl);

        //find the node we're after
        Node node = page.getDocumentElement();
        
        //create a view on the node
        TemplateView tv = null;
        if (useIDTemplate) {
            String propFileName = "HelloWorld3a.properties";
            Properties props = new Properties();
            try {
                props.load(this.getClass().getResourceAsStream(propFileName));
                tv = new DefaultTemplateView(node, "id", new MapStateMap(props));        
            } catch (IOException e) {
                System.out.println ("Fatal err loading properties file:"+e);
            }
        } else {
            tv = new DefaultTemplateView(node, "tid", "tdir");
        }

        //create the component and bind it to the view
        BTemplate templateComp = new BTemplate();
        templateComp.setView(tv);

        //now specify the model
        templateComp.addModel(new HelloWorldModel(useIDTemplate));
        String[] items = new String[] {"Cabbage", "Carrots", "Pickles", "Sugar", "Wheaties", "Flour", "Potatoes"};
        String[] qtys =  new String[] {"3 heads", "1 bag", "1 jar", "5 lbs", "3 bxs", "50 lbs", "25 lbs"};
        templateComp.addModel(new GroceriesModel(items, qtys));
        
        //add the component to the root
        root.addChild(templateComp);
    
        //return the page
        return page;
    }

    //--------------- GroceriesModel -------------------------
    /**
     * Groceries Model
     */
    class GroceriesModel extends AbstractTemplateModel implements IterativeModel {

        String[] items = null;
        String[] qtys = null;
        int cntr = -1;

        //constructor
        public GroceriesModel(String[] iitems, String[] iqtys) {
            items = iitems;
            qtys = iqtys;
        }

        //register the model by name
        public String getName() {return "Groceries";}

        //provide items by key
        public Object getItem(String key) {
            if (key.equals("Item")) return items[cntr];
            else if (key.equals("Qty")) return qtys[cntr];
            else return super.getItem(key);
        }

        //prepare for iteration
        public void preIterate() {
            cntr = -1;
        }
        
        //return true if there are more rows in the data model
        public boolean hasNext() {
            return (cntr<(items.length-1));
        }
        
        //load the next row in the model
        public void loadNext() {
            cntr++;
        }

        //cleanup after iteration
        public void postIterate() {}
    }

    //--------------- HelloWorldModel ------------------------
    /**
     * HelloWorldModel
     */
    class HelloWorldModel extends AbstractTemplateModel {
    
        String source = null;
        
        public HelloWorldModel(boolean useIDTemplate) {
            if (useIDTemplate) source = "Here the template processing instructions are stored in a separate .properties file and just referenced using the tid attribute.";
            else source = "Here the template processing instructions are actually stored in the template file as directives embedded in the tdir attributes.";
        }
    
        //register the model by name
        public String getName() {
            return "HelloWorld";
        }
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            if (key.equals("Title")) return "Hello World 3 example";
            else if (key.equals("Descr")) return "This example shows BTemplate processing an XML template. "+source;
            else if (key.equals("Hello")) return "Hello World! Hi Ma! Hi Pa!";
            else return super.getItem(key);
        }
    }
}
