/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestDateRangeValidator.java,v 1.8 2002/06/11 01:19:02 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.io.*;
import java.util.*;
import java.text.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.data.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.util.DateUtil;
import org.enhydra.barracuda.testbed.*;

/**
 * This test verifies that DateRangeValidator works correctly.
 */
public class TestDateRangeValidator extends ValidatorTestCase {

	//common vars (customize for every test class)
	private static String testClass = TestDateRangeValidator.class.getName();
	private static Logger logger = Logger.getLogger("test."+testClass);

	//variables
	
	//-------------------- Basics --------------------------------

	/**
	 * Public Constructor
	 */
	public TestDateRangeValidator(String name) {
		super(name);
	}
	
	/**
	 * Every test class should have a main method so it can be run
	 * directly (when debugging tests you often will not want to run
	 * the whole suite)
	 *
	 * @param args defined in test.util.TestUtil
	 */
	public static void main(String args[]) {
		//check for standard runtime parameters
		TestUtil.parseParams(args);

		//launch the test
		if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
		else junit.swingui.TestRunner.main(new String[] {testClass});
	}

	public static Test suite() {

		TestSuite allLocalesSuite = new TestSuite();

		Locale[] availableLocales = Locale.getAvailableLocales();
		for (int i = 0; i < availableLocales.length; i++) {
			TestSuite suite = new TestSuite();
			suite.addTestSuite( TestDateRangeValidator.class );
			allLocalesSuite.addTest( new LocaleTestSuiteSetup( suite, availableLocales[i]) );
			logger.info("Adding:" + availableLocales[i].getCountry() );
		}

		return allLocalesSuite;
	}
	
	//-------------------- Actual Tests --------------------------
	//Note: all the methods herein should follow the testXXXX naming convention
	//Also keep in mind that local vars set in one test method are NOT retained
	//when the next method is invoked because JUnit makes a separate instance of
	//the test class for each testXXXX method!!!
	
	/**
	 * Test String types
	 */
	public void testString() {
		DateFormat aDateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
		aDateFormat.setLenient(false);
		DefaultFormElement el = new DefaultFormElement("key", FormType.STRING);

		Date now = new Date();
		DateRangeValidator afterNowValidator = new DateRangeValidator(now,null);

    // ilc_022202.1_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("StringAfterNowInvalid1", "foo");
    sm.putState("StringAfterNowInvalid2", TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"));
    sm.putState("StringAfterNowInvalid3", TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"));
    sm.putState("StringAfterNowInvalid4", TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"));
    // now should pass a range of today..null
    //new String( aDateFormat.format( now ) ),
    sm.putState("s2", new String( aDateFormat.format( DateUtil.getLastDateYesterday().getTime())));

    this.assertAllInvalid(afterNowValidator, sm, FormType.STRING);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("StringAfterNowValid1", null);
    sm.putState("StringAfterNowValid2", "");
    sm.putState("StringAfterNowValid3", TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1"));
    sm.putState("StringAfterNowValid4", TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"));
    sm.putState("StringAfterNowValid5", TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31"));
    sm.putState("StringAfterNowValid6", new String( aDateFormat.format( DateUtil.getFirstDateTomorrow().getTime())));

    this.assertAllValid(afterNowValidator, sm, FormType.STRING);



    DateRangeValidator beforeNowValidator = new DateRangeValidator(null,now);

    sm = null;
    sm = new DefaultStateMap();
    //isNull test strips characters
    //sm.putState("StringBeforeNowInValid1", "   ");
    sm.putState("StringBeforeNowInValid1", "foo");
    sm.putState("StringBeforeNowInValid2", TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1"));
    sm.putState("StringBeforeNowInValid3", TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"));
    sm.putState("StringBeforeNowInValid4", TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31"));
    sm.putState("StringBeforeNowInValid5", new String( aDateFormat.format( DateUtil.getFirstDateTomorrow().getTime())));

    this.assertAllInvalid(beforeNowValidator, sm, FormType.STRING);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("StringBeforeNowValid1", null);
    sm.putState("StringBeforeNowValid2", "");
    sm.putState("StringBeforeNowValid3", TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"));
    sm.putState("StringBeforeNowValid4", TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"));
    sm.putState("StringBeforeNowValid5", TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"));
    sm.putState("StringBeforeNowValid6", new String( aDateFormat.format( now )));
    sm.putState("StringBeforeNowValid7", new String( aDateFormat.format( DateUtil.getLastDateYesterday().getTime())));

    this.assertAllValid(beforeNowValidator, sm, FormType.STRING);



    DateRangeValidator todayValidator = new DateRangeValidator( DateUtil.getFirstDateToday().getTime(),
                                                                DateUtil.getLastDateToday().getTime() );
    sm = null;
    sm = new DefaultStateMap();
    //isNull test strips characters
    //sm.putState("StringTodayInvalid1", "   ");
    sm.putState("StringTodayInvalid1", "foo");
    sm.putState("StringTodayInvalid2", TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"));
    sm.putState("StringTodayInvalid3", TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"));
    sm.putState("StringTodayInvalid4", TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"));
    sm.putState("StringTodayInvalid5", TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"));
    sm.putState("StringTodayInvalid6", new String( aDateFormat.format( DateUtil.getLastDateYesterday().getTime())));
    sm.putState("StringTodayInvalid7", new String( aDateFormat.format( DateUtil.getFirstDateTomorrow().getTime())));

    this.assertAllInvalid(todayValidator, sm, FormType.STRING);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("StringTodayValid1", null);
    sm.putState("StringTodayValid2", "");
    sm.putState("StringTodayValid3", new String( aDateFormat.format( new Date())));
    sm.putState("StringTodayValid4", new String( aDateFormat.format( DateUtil.getFirstDateToday().getTime())));
    sm.putState("StringTodayValid5", new String( aDateFormat.format( DateUtil.getLastDateToday().getTime())));

    this.assertAllValid(todayValidator, sm, FormType.STRING);


    // ilc_022202.1_end

    // ilc_022202.2_start
    // commented out to use statemaps for validation
    /*
    this.assertAllInvalid(afterNowValidator, el,
									 new String[]{
                      // ilc_022102.1_start
                      // comment out failing blanks
                      //"   ",
                      // ilc_022102.1_end
											"foo",
											TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"),
											TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"),
											TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"),
                      // ilc_022102.2_start
                      // now should pass a range of today..null
                      //new String( aDateFormat.format( now ) ),
                      // ilc_022102.2_end
											new String( aDateFormat.format( DateUtil.getLastDateYesterday().getTime() ) ),

														 } );
    this.assertAllValid(afterNowValidator, el,
									 new String[]{
											null,
											"",
											TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1"),
											TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"),
											TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31"),
											new String( aDateFormat.format( DateUtil.getFirstDateTomorrow().getTime() ) ),
														 } );

		DateRangeValidator beforeNowValidator = new DateRangeValidator(null,now);

    this.assertAllInvalid(beforeNowValidator, el,
										 new String[]{
                      // ilc_022102.3_start
                      // comment out failing blanks
                      //"   ",
                      // ilc_022102.3_end
												"foo",
												TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1"),
												TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"),
												TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31"),
												new String( aDateFormat.format( DateUtil.getFirstDateTomorrow().getTime() ) ),
														 } );
    this.assertAllValid(beforeNowValidator, el,
									 new String[]{
											null,
											"",
											TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"),
											TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"),
											TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"),
											new String( aDateFormat.format( now ) ),
											new String( aDateFormat.format( DateUtil.getLastDateYesterday().getTime() ) ),
														 } );

		DateRangeValidator todayValidator = new DateRangeValidator( DateUtil.getFirstDateToday().getTime(),
																																DateUtil.getLastDateToday().getTime() );

    this.assertAllInvalid(todayValidator, el,
									 new String[]{
                      // ilc_022102.4_start
                      // comment out failing blanks
                      //"   ",
                      // ilc_022102.4_end
											"foo",
											TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"),
											TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"),
											TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"),
											TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"),
											new String( aDateFormat.format( DateUtil.getLastDateYesterday().getTime() ) ),
											new String( aDateFormat.format( DateUtil.getFirstDateTomorrow().getTime() ) ),
														 } );

    this.assertAllValid(todayValidator, el,
									 new String[]{
											null,
											"",
											new String( aDateFormat.format( new Date() ) ),
											new String( aDateFormat.format( DateUtil.getFirstDateToday().getTime() ) ),
											new String( aDateFormat.format( DateUtil.getLastDateToday().getTime() ) ),
														 } );
    */
    // ilc_022202.2_end
	}

	public void testLong() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.LONG);

    // ilc_022202.3_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("LongInvalid1", new Long(0));
    sm.putState("LongInvalid2", new Long(1));
    sm.putState("LongInvalid3", new Long(-1));
    sm.putState("LongInvalid4", new Long(Long.MIN_VALUE));
    sm.putState("LongInvalid5", new Long(Long.MAX_VALUE));

    this.assertAllInvalid(v, sm, FormType.LONG);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("LongValid1", null);
    this.assertAllValid(v, sm, FormType.LONG);

    // ilc_022202.3_end

    // ilc_022202.4_start
    // commented out to use state map
    /*
    this.assertAllInvalid(v, el, new Long[]{ new Long(0),
																				new Long(1),
																				new Long(-1),
																				new Long(Long.MIN_VALUE),
																				new Long(Long.MAX_VALUE),
																			} );

    this.assertAllValid(v, el, new Long[]{ null, } );
    */
    //ilc_022202.4_end
	}
	
	public void testBoolean() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.BOOLEAN);

    // ilc_022202.5_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("BooleanInvalid1", Boolean.FALSE);
    sm.putState("BooleanInvalid2", Boolean.TRUE);

    this.assertAllInvalid(v, sm, FormType.BOOLEAN);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("BooleanValid1", null);
    this.assertAllValid(v, sm, FormType.BOOLEAN);

    // ilc_022202.5_end

    // ilc_022202.6_start
    /* commented out to use statemaps instead
    this.assertAllInvalid(v, el, new Boolean[] { Boolean.FALSE,
																						Boolean.FALSE,
																					} );

    this.assertAllValid(v, el, new Boolean[]{ null, } );
    */
    // ilc_022202.6_end
	}
	
	public void testShort() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.SHORT);

    // ilc_022202.7_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("ShortInvalid1", new Short((short)0));
    sm.putState("ShortInvalid2", new Short((short)1));
    sm.putState("ShortInvalid3", new Short((short)-1));
    sm.putState("ShortInvalid4", new Short(Short.MIN_VALUE));
    sm.putState("ShortInvalid5", new Short(Short.MAX_VALUE));


    this.assertAllInvalid(v, sm, FormType.SHORT);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("ShortValid1", null);
    this.assertAllValid(v, sm, FormType.SHORT);

    // ilc_022202.7_end

    // ilc_022202.8_start
    /* commented out to use statemaps instead
    this.assertAllInvalid(v, el, new Short[] { new Short((short)0),
																					new Short((short)1),
																					new Short((short)-1),
																					new Short(Short.MIN_VALUE),
																					new Short(Short.MAX_VALUE),
																				} );

    this.assertAllValid(v, el, new Short[]{ null, } );
    */
    // ilc_022202.8_end

	}
	
	public void testInteger() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.INTEGER);
    // ilc_022202.9_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("IntegerInvalid1", new Integer(0));
    sm.putState("IntegerInvalid2", new Integer(1));
    sm.putState("IntegerInvalid3", new Integer(-1));
    sm.putState("IntegerInvalid4", new Integer(Integer.MIN_VALUE));
    sm.putState("IntegerInvalid5", new Integer(Integer.MAX_VALUE));

    this.assertAllInvalid(v, sm, FormType.INTEGER);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("IntegerValid1", null);
    this.assertAllValid(v, sm, FormType.INTEGER);

    // ilc_022202.9_end

    // ilc_022202.10_start
    /* commented out to use statemaps instead
    this.assertAllInvalid(v, el, new Integer[] { new Integer(0),
																						new Integer(1),
																						new Integer(-1),
																						new Integer(Integer.MIN_VALUE),
																						new Integer(Integer.MAX_VALUE),
																					} );

    this.assertAllValid(v, el, new Integer[]{ null, } );
    */
    // ilc_022202.10_end
	}
	
	public void testDate()  {
		DateFormat aDateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
		aDateFormat.setLenient(false);
		DefaultFormElement el = new DefaultFormElement("key", FormType.DATE);

		Date now = new Date();
		DateRangeValidator afterNowValidator = new DateRangeValidator(now,null);

    // ilc_022202.11_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    try {
      sm.putState("DateAfterNowInvalid1", new Date(now.getTime()-1));
      sm.putState("DateAfterNowInvalid2", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")));
      sm.putState("DateAfterNowInvalid3", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")));
      sm.putState("DateAfterNowInvalid4", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")));

      this.assertAllInvalid(afterNowValidator, sm, FormType.DATE);

      sm = null;
      sm = new DefaultStateMap();
      sm.putState("DateAfterNowValid1", null);
      sm.putState("DateAfterNowValid2", new Date());
      sm.putState("DateAfterNowValid3", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1")));
      sm.putState("DateAfterNowValid4", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")));
      sm.putState("DateAfterNowValid5", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31")));

      this.assertAllValid(afterNowValidator, sm, FormType.DATE);



      DateRangeValidator beforeNowValidator = new DateRangeValidator(null,now);

      sm = null;
      sm = new DefaultStateMap();
      sm.putState("DateBeforeNowInValid1", new Date(now.getTime()+1));
      sm.putState("DateBeforeNowInValid2", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1")));
      sm.putState("DateBeforeNowInValid3", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")));
      sm.putState("DateBeforeNowInValid4", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31")));

      this.assertAllInvalid(beforeNowValidator, sm, FormType.DATE);

      sm = null;
      sm = new DefaultStateMap();
      sm.putState("DateBeforeNowValid1", null);
      sm.putState("DateBeforeNowValid2", new Date(now.getTime()-1));
      sm.putState("DateBeforeNowValid3", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")));
      sm.putState("DateBeforeNowValid4", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")));
      sm.putState("DateBeforeNowValid5", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")));

      this.assertAllValid(beforeNowValidator, sm, FormType.DATE);



      DateRangeValidator todayValidator = new DateRangeValidator( DateUtil.getFirstDateToday().getTime(),
                                                                  DateUtil.getLastDateToday().getTime() );
      sm = null;
      sm = new DefaultStateMap();
      sm.putState("DateTodayInvalid1", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")));
      sm.putState("DateTodayInvalid2", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")));
      sm.putState("DateTodayInvalid3", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")));;
      sm.putState("DateTodayInvalid4", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")));
      sm.putState("DateTodayInvalid5", DateUtil.getLastDateYesterday().getTime());
      sm.putState("DateTodayInvalid6", DateUtil.getFirstDateTomorrow().getTime());

      this.assertAllInvalid(todayValidator, sm, FormType.DATE);

      sm = null;
      sm = new DefaultStateMap();
      sm.putState("DateTodayValid1", null);
      sm.putState("DateTodayValid2", new Date());
      sm.putState("DateTodayValid3", DateUtil.getFirstDateToday().getTime());
      sm.putState("DateTodayValid4", DateUtil.getLastDateToday().getTime());

      this.assertAllValid(todayValidator, sm, FormType.DATE);

    } catch(java.text.ParseException ex) {
      fail( "testDate failed because " + ex.getMessage() );
    }
    // ilc_022202.11_end

    // ilc_022202.12_start
    /* commented out to use statemaps instead
		try {
      this.assertAllInvalid(afterNowValidator, el,
										 new Date[]{
												new Date(now.getTime()-1),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")),
															 } );
      this.assertAllValid(afterNowValidator, el,
										 new Date[]{
												null,
												new Date(),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31")),
															 } );
		} catch ( java.text.ParseException ex )  {
			fail( "testDate failed because " + ex.getMessage() );
		}

		DateRangeValidator beforeNowValidator = new DateRangeValidator(null,now);

		try {
      this.assertAllInvalid(beforeNowValidator, el,
										 new Date[]{
												new Date(now.getTime()+1),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31")),
															 } );
      this.assertAllValid(beforeNowValidator, el,
										 new Date[]{
												null,
												new Date(now.getTime()-1),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")),
															 } );
		} catch ( java.text.ParseException ex )  {
			fail( "testDate failed because " + ex.getMessage() );
		}

		DateRangeValidator todayValidator = new DateRangeValidator( DateUtil.getFirstDateToday().getTime(),
																																DateUtil.getLastDateToday().getTime() );
		try {
      this.assertAllInvalid(todayValidator, el,
										 new Date[]{
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")),
												DateUtil.getLastDateYesterday().getTime(),
												DateUtil.getFirstDateTomorrow().getTime(),
															 } );
      this.assertAllValid(todayValidator, el,
										 new Date[]{
												null,
												new Date(),
												DateUtil.getFirstDateToday().getTime(),
												DateUtil.getLastDateToday().getTime(),
															 } );
		} catch ( java.text.ParseException ex )  {
			fail( "testDate failed because " + ex.getMessage() );
		}
    */
    // ilc_022202.12_end

	}
	
	public void testDouble() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.DOUBLE);
    // ilc_022202.13_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("DoubleInvalid1", new Double(0));
    sm.putState("DoubleInvalid2", new Double(1));
    sm.putState("DoubleInvalid3", new Double(-1));
    sm.putState("DoubleInvalid4", new Double(Double.MIN_VALUE));
    sm.putState("DoubleInvalid5", new Double(Double.MAX_VALUE));

    this.assertAllInvalid(v, sm, FormType.DOUBLE);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("DoubleValid1", null);
    this.assertAllValid(v, sm, FormType.DOUBLE);

    // ilc_022202.13_end

    // ilc_022202.14_start
    /* commented out to use statemaps instead
    this.assertAllInvalid(v, el, new Double[] {  new Double(0),
																						new Double(1),
																						new Double(-1),
																						new Double(Double.MIN_VALUE),
																						new Double(Double.MAX_VALUE),
																				 } );

    this.assertAllValid(v, el, new Double[]{ null, } );
    */
    // ilc_022202.14_end
	}
	
	public void testFloat() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.FLOAT);

    // ilc_022202.15_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("FloatInvalid1", new Float(0));
    sm.putState("FloatInvalid2", new Float(1));
    sm.putState("FloatInvalid3", new Float(-1));
    sm.putState("FloatInvalid4", new Float(Float.MIN_VALUE));
    sm.putState("FloatInvalid5", new Float(Float.MAX_VALUE));

    this.assertAllInvalid(v, sm, FormType.FLOAT);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("FloatValid1", null);
    this.assertAllValid(v, sm, FormType.FLOAT);

    // ilc_022202.15_end

    // ilc_022202.16_start
    /* commented out to use statemaps instead
    this.assertAllInvalid(v, el, new Float[] { new Float(0),
																					new Float(1),
																					new Float(-1),
																					new Float(Float.MIN_VALUE),
																					new Float(Float.MAX_VALUE),
																				} );

    this.assertAllValid(v, el, new Float[]{ null, } );
    */
    // ilc_022202.16_end
	}

}
