/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestFormatType.java,v 1.5 2002/05/29 06:17:36 JacobK Exp $
 */
package org.enhydra.barracuda.core.view;

import java.io.*;
import java.util.*;
import java.security.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.*;
import org.enhydra.barracuda.testbed.*;
import org.enhydra.barracuda.testbed.servlet.*;


/**
 * This test verifies that the hierachical organization of 
 * FormatType classes. We do this by testing for what it should be 
 * an instanceof and what it shouldn't. The idea is that the FormatType
 * extension hierarchy is very important and we don't want it getting 
 * changed accidentally. This test should preclude that.
 */
public class TestFormatType extends TestCase {
    //common vars (customize for every test class)
    private static String testClass = TestFormatType.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables
    FormatType ft = null;
    Class targetCl = null;
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestFormatType(String name) {
        super(name);
    }

    public TestFormatType(String name, FormatType ift, Class itargetCl) {
        super(name);
        ft = ift;
        targetCl = itargetCl;
    }
    
    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    /**
     * Return a suite of tests...
     */
	public static Test suite() {
        //create the test suite
		TestSuite suite = new TestSuite();
        
        //add the tests
        //...HTML_3_0
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_0, FormatType.Html3x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_0, FormatType.Html.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_0, FormatType.Html4x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_0, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_0, FormatType.UnknownFormat.class));
        //...HTML_3_1
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_1, FormatType.Html3x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_1, FormatType.Html.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_1, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_1, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_1, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_1, FormatType.Html4x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_1, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_1, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_1, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_1, FormatType.UnknownFormat.class));
        //...HTML_3_2
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_2, FormatType.Html3x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_2, FormatType.Html.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_2, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_2, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_2, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_2, FormatType.Html4x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_2, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_2, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_2, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_2, FormatType.UnknownFormat.class));
        //...HTML_4_0
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.Html4x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.Html3x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.Html.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_0, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_0, FormatType.UnknownFormat.class));
        //...HTML_4_1
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.Html4x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.Html3x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.Html.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_1, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_1, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_1, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_1, FormatType.UnknownFormat.class));
        //...CHTML_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_1_0, FormatType.Chtml1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_1_0, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_1_0, FormatType.Chtml2x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_1_0, FormatType.Html.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_1_0, FormatType.UnknownFormat.class));
        //...CHTML_2_0
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.Chtml2x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.Chtml1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_2_0, FormatType.Html.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_2_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_2_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_2_0, FormatType.UnknownFormat.class));
        //...XML_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.XML_1_0, FormatType.Xml1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XML_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XML_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XML_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.Vxml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.Wml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.Xhtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.UnknownFormat.class));
        //...VXML_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.VXML_1_0, FormatType.Vxml1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.VXML_1_0, FormatType.Vxml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.VXML_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.VXML_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.VXML_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.Xml1x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.Wml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.Xhtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.UnknownFormat.class));
        //...WML_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.WML_1_0, FormatType.Wml1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.WML_1_0, FormatType.Wml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.WML_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.WML_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.WML_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.Xml1x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.Vxml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.Xhtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.UnknownFormat.class));
        //...XHTML_BASIC_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.XhtmlBasic1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.XhtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.Xhtml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.XhtmlBasic2x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.XhtmlStandard.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.Xml1x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.Vxml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.Wml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.UnknownFormat.class));
        //...XHTML_STANDARD_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlStandard1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlStandard.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlBasic1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.Xhtml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlStandard2x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlBasic2x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.Xml1x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.Vxml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.Wml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.UnknownFormat.class));
        //...UNKNOWN_FORMAT
        suite.addTest(new TestFormatType("testInstance", FormatType.UNKNOWN_FORMAT, FormatType.UnknownFormat.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.UNKNOWN_FORMAT, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.UNKNOWN_FORMAT, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.UNKNOWN_FORMAT, FormatType.BinaryData.class));

        //finally return the suite
        return suite;
    }


    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    /**
     * Make sure that the class being tested IS an instance of the 
     * target class
     */
    public void testInstance() {
        if (logger.isInfoEnabled()) logger.info("testing instanceof");
        assertTrue(ft+" not an instanceof "+targetCl, targetCl.isAssignableFrom(ft.getClass()));
    }

    /**
     * Make sure that the class being tested is NOT an instance of the 
     * target class
     */
    public void testNotInstance() {
        if (logger.isInfoEnabled()) logger.info("testing !instanceof");
        assertTrue(ft+" is an instanceof "+targetCl+" (and it shouldn't be!)", !targetCl.isAssignableFrom(ft.getClass()));
    }
}
